﻿/*
*	FILE			: ESMIS_CLIENT.cs
*	PROJECT			: PROG2030 - Final Project
*	PROGRAMMER		: James Simpson, Brian Hinds, Jackson Ruby
*	FIRST VERSION	: 
*	DESCRIPTION 	: Functions dealing with user interactions
*/
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Data;
using System.Net.Sockets;
using System.Windows.Forms;

namespace ESMIS
{
    public partial class ESMIS_CLIENT : Form
    {
        /// <summary>
        /// 
        /// </summary>
        private Hashtable esmis_stocks = new Hashtable();
        private ESMIS_LOGIN login = new ESMIS_LOGIN();
        private string key;
        private string val;
        private string selected_sysmbol;
        private bool basic = true;
        private bool energy = true;
        private bool finance = true;
        private bool health = true;
        private bool tech = true;
        private bool transport = true;
        private bool utilities = true;
        private ESMIS_GET_STOCK getStock = new ESMIS_GET_STOCK();
        private string getStockResults;
        private string stockRange;
        private int count = 0;
        private string selectedSymbol;
        private double selectedSymbolAsk;
        /// <summary>
        /// 
        /// </summary>
        private string userData = ESMIS_LOGIN.userProfile;//user profile
        private  string userName = "";
        private string company = "";
        private string stockAmount = "";
        private string transaction = "";
        private string transactionType = "";
        private string transactionDate = "";
        private string bankBalance = "";
        private string role = "";

        //For user stock info/profile
        private string UserName;
        public double userBankBalance = 100000;
        List<string> CompanySymbols = new List<string>();
        List<int> StocksOwned = new List<int>();

        //Lists for (displaying) transaction information
        List<string> CompanySymbolsHistory = new List<string>();
        List<int> stockTranacrtionHistory = new List<int>();
        List<string> TransactionAmountsHistory = new List<string>();
        List<string> TransactionTypesHistory = new List<string>();
        List<string> TransactionDatesHistory = new List<string>();

        public ESMIS_CLIENT()
        {
            InitializeComponent();
            load_stocks();
            bankBalanceTB.Items.Add(userBankBalance.ToString());
            buy_button.Enabled = false;
            sell_button.Enabled = false;
            short_sell_button.Enabled = false;
            short_cover_button.Enabled = false;
            numb_of_shares_box.Enabled = false;
            limit_price_box.Enabled = false;
            userProfileExtract();
            loadTransactions();
        }

        /*
       *  FUNCTION       : load_stocks()
       *  DISCRIPTION    : Loads the accessible stock symbols for client
       *  PARAMETER      : None
       *  RETURN         : None
       */

        private void load_stocks()
        {
            if (basic)
            {
                List<string> readLines = File.ReadAllLines("basic.esmis").ToList();
                foreach (string line in readLines)
                {
                    key = line.Split('\t')[0];
                    val = line.Split('\t')[1];

                    if (!esmis_stocks.ContainsKey(key))
                    {
                        esmis_stocks.Add(key, val);
                    }
                }
            }

            if (energy)
            {
                List<string> readLines = File.ReadAllLines("energy.esmis").ToList();
                foreach (string line in readLines)
                {
                    key = line.Split('\t')[0];
                    val = line.Split('\t')[1];

                    if (!esmis_stocks.ContainsKey(key))
                    {
                        esmis_stocks.Add(key, val);
                    }
                }
            }

            if (finance)
            {
                List<string> readLines = File.ReadAllLines("finance.esmis").ToList();
                foreach (string line in readLines)
                {
                    key = line.Split('\t')[0];
                    val = line.Split('\t')[1];

                    if (!esmis_stocks.ContainsKey(key))
                    {
                        esmis_stocks.Add(key, val);
                    }
                }
            }

            if (health)
            {
                List<string> readLines = File.ReadAllLines("health.esmis").ToList();
                foreach (string line in readLines)
                {
                    key = line.Split('\t')[0];
                    val = line.Split('\t')[1];

                    if (!esmis_stocks.ContainsKey(key))
                    {
                        esmis_stocks.Add(key, val);
                    }
                }
            }

            if (tech)
            {
                List<string> readLines = File.ReadAllLines("tech.esmis").ToList();
                foreach (string line in readLines)
                {
                    key = line.Split('\t')[0];
                    val = line.Split('\t')[1];

                    if (!esmis_stocks.ContainsKey(key))
                    {
                        esmis_stocks.Add(key, val);
                    }
                }
            }

            if (transport)
            {
                List<string> readLines = File.ReadAllLines("transport.esmis").ToList();
                foreach (string line in readLines)
                {
                    key = line.Split('\t')[0];
                    val = line.Split('\t')[1];

                    if (!esmis_stocks.ContainsKey(key))
                    {
                        esmis_stocks.Add(key, val);
                    }
                }
            }

            if (utilities)
            {
                List<string> readLines = File.ReadAllLines("utilities.esmis").ToList();
                foreach (string line in readLines)
                {
                    key = line.Split('\t')[0];
                    val = line.Split('\t')[1];

                    if (!esmis_stocks.ContainsKey(key))
                    {
                        esmis_stocks.Add(key, val);
                    }
                }
            }
        }

        /*
       *  FUNCTION       : search_button_Click
       *  DISCRIPTION    : Search button
       *  PARAMETER      : (object sender, EventArgs e)
       *  RETURN         : None
       */

        private void search_stocks()
        {
            if (esmis_stocks.ContainsKey(search_box.Text))
            {
                foreach (DictionaryEntry symbols in esmis_stocks)
                    if (search_box.Text == Convert.ToString(symbols.Key))
                    {
                        symbol_results.Items.Add(symbols.Value);
                    }
            }
            else
            {
                MessageBox.Show("Not Found");
            }
        }

        /*
       *  FUNCTION       : search_button_Click
       *  DISCRIPTION    : Search button
       *  PARAMETER      : (object sender, EventArgs e)
       *  RETURN         : None
       */

        private void search_button_Click(object sender, EventArgs e)
        {
            search_stocks();
        }

        /*
       *  FUNCTION       : symbol_results_SelectedIndexChanged
       *  DISCRIPTION    : Allow selected symbol to be searched
       *  PARAMETER      : (object sender, EventArgs e)
       *  RETURN         : None
       */

        private void symbol_results_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Enable buttons and boxes for buying and selling
            buy_button.Enabled = true;
            sell_button.Enabled = true;
            numb_of_shares_box.Enabled = true;

            count = 0;
            try
            {
                selected_sysmbol = symbol_results.SelectedItem.ToString();
                if (selected_sysmbol != null)
                {
                    bool companyAlreadySearched = false;
                    //Searching the hash table for the selected company symbol.
                    foreach (DictionaryEntry symbols in esmis_stocks)
                        if (selected_sysmbol == Convert.ToString(symbols.Value))
                        {
                            selected_sysmbol = Convert.ToString(symbols.Key);

                            //Stop items from appearing in Stock Info more than once
                            for (int i = 0; i < selected_company.Items.Count; i++)
                            {
                                if (selected_company.Items[i].ToString().Equals(symbol_results.SelectedItem.ToString()))
                                {
                                    companyAlreadySearched = true;
                                }
                            }

                            //Add company if it hasn't been selected yet
                            if (companyAlreadySearched == false)
                            {
                                selected_company.Items.Add(symbol_results.SelectedItem.ToString());
                            }
                            //Pull stock results.
                            getStockResults = getStock.getStockQuote(selected_sysmbol);
                            //Get the values for the chart
                            stockRange = getStock.getStockRange(selected_sysmbol);
                            //Set delimiter for the stock results
                            char delimiter = ',';
                            //Set up a string array for the stock results
                            string[] substrings = getStockResults.Split(delimiter);
                            //loop through the string and assign values to list boxes.
                            if (companyAlreadySearched == false)
                            {
                                foreach (var substring in substrings)
                                {
                                    if (count == 0) { s_Date.Items.Add(substring); }
                                    if (count == 1) { s_Open.Items.Add("$" + substring); }
                                    if (count == 2) { s_High.Items.Add("$" + substring); }
                                    if (count == 3) { s_Low.Items.Add("$" + substring); }
                                    if (count == 4) { s_Close.Items.Add("$" + substring); }
                                    if (count == 5) { s_Volume.Items.Add("$" + substring); }
                                    count++;
                                }
                            }
                            updateStockChart();
                        }
                }
            }
            catch (Exception ex)
            {

            }
        }


        /*
       *  FUNCTION       : updateStockChart()
       *  DISCRIPTION    : updates a chart with information about a selected stock
       *  PARAMETER      : none
       *  RETURN         : none
       */

        private void updateStockChart()
        {
            stock_chart.Series.Clear();

            //For the split string
            double lowVlaue = 0;
            double highValue = 0;
            double askValue = 0;
            double average50day = 0;
            double average200day = 0;

            //Split the range string
            char delimiter = ',';
            string[] substrings = stockRange.Split(delimiter);

            //Split the values in the chart
            lowVlaue = Convert.ToDouble(substrings[0]);
            highValue = Convert.ToDouble(substrings[1]);
            askValue = Convert.ToDouble(substrings[2]);
            selectedSymbolAsk = Convert.ToDouble(substrings[2]);
            average50day = Convert.ToDouble(substrings[3]);
            average200day = Convert.ToDouble(substrings[4]);
            selectedSymbol = Convert.ToString(substrings[5]);
            stock_chart.Series.Add("StockData for " + selectedSymbol);

            //Set the maximum y value on the chart to slightly more than the 52 week high value
            stock_chart.ChartAreas[0].AxisY.Maximum = highValue + (highValue / 4);
            count = 0;

            //Set chart type to line
            stock_chart.Series[0].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.Column;


            //Set points on chart and label them
            stock_chart.Series[0].Points.AddY(lowVlaue);
            stock_chart.Series[0].Points[0].AxisLabel = "52 Week Low";
            stock_chart.Series[0].Points.AddY(highValue);
            stock_chart.Series[0].Points[1].AxisLabel = "52 Week High";
            stock_chart.Series[0].Points.AddY(askValue);
            stock_chart.Series[0].Points[2].AxisLabel = "Current Ask Value";
            stock_chart.Series[0].Points.AddY(average50day);
            stock_chart.Series[0].Points[3].AxisLabel = "50 Day Average";
            stock_chart.Series[0].Points.AddY(average200day);
            stock_chart.Series[0].Points[4].AxisLabel = "200 Day Average";
            stock_chart.ChartAreas[0].AxisY.Title = "Prics ($)";
        }

        /*
       *  FUNCTION       : contactUs_Click
       *  DISCRIPTION    : Contact form
       *  PARAMETER      : (object sender, EventArgs e)
       *  RETURN         : None
       */

        private void contactUs_Click(object sender, EventArgs e)
        {

        }
        /*
        *  FUNCTION       : buy_button_Click()
        *  DISCRIPTION    : allows buying of stocks
        *  PARAMETER      : 
        *  RETURN         : none
        */
        private void buy_button_Click(object sender, EventArgs e)
        {
            try
            {
                double priceToPurchase = Convert.ToInt32(numb_of_shares_box.Text) * selectedSymbolAsk;
                if (priceToPurchase > userBankBalance)
                {
                    MessageBox.Show("You do not have enough money to purchase " +
                        numb_of_shares_box.Text + " shares");
                }
                else
                {
                    userBankBalance -= priceToPurchase;
                    bankBalanceTB.Items[0] = userBankBalance.ToString();

                    //First time purchasing stock
                    if (!CompanySymbols.Contains(selectedSymbol))
                    {

                        CompanySymbols.Add(selectedSymbol);
                        StocksOwned.Add(Convert.ToInt32(numb_of_shares_box.Text));
                    }

                    //User has already bought this stock before
                    else if (CompanySymbols.Contains(selectedSymbol))
                    {
                        int index = CompanySymbols.IndexOf(selectedSymbol);
                        StocksOwned[index] += Convert.ToInt32(numb_of_shares_box.Text);
                    }

                    //Add to display lists
                    CompanySymbolsHistory.Add(selectedSymbol);
                    stockTranacrtionHistory.Add(Convert.ToInt32(numb_of_shares_box.Text));
                    TransactionAmountsHistory.Add("$" + priceToPurchase.ToString());
                    TransactionTypesHistory.Add("BUY");
                    TransactionDatesHistory.Add(DateTime.Now.ToString());

                    //Update transaction listBoxes
                    updateTransactions();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Number of shares not specified");
            }
        }
        /*
        *  FUNCTION       : sell_button_Click()
        *  DISCRIPTION    : allows the user to sell stocks
        *  PARAMETER      : 
        *  RETURN         : none
        */
        private void sell_button_Click(object sender, EventArgs e)
        {
            try
            {
                bool hasStock = checkForStock();
                bool hasAmount = false;
                if (hasStock == true)
                {
                    hasAmount = checkForStockAmount(Convert.ToInt32(numb_of_shares_box.Text));
                }
                double SalesEarnings = Convert.ToInt32(numb_of_shares_box.Text) * selectedSymbolAsk;

                if (hasStock == true && hasAmount == true)
                {
                    userBankBalance += SalesEarnings;
                    bankBalanceTB.Items[0] = userBankBalance.ToString();

                    //Update user lists
                    int index = CompanySymbols.IndexOf(selectedSymbol);
                    StocksOwned[index] -= Convert.ToInt32(numb_of_shares_box.Text);

                    //Add to display lists and history
                    CompanySymbolsHistory.Add(selectedSymbol);
                    stockTranacrtionHistory.Add(Convert.ToInt32(numb_of_shares_box.Text));
                    TransactionAmountsHistory.Add("$" + SalesEarnings.ToString());
                    TransactionTypesHistory.Add("SALE");
                    TransactionDatesHistory.Add(DateTime.Now.ToString());

                    //Update transaction listBoxes
                    updateTransactions();
                }
                else if (hasStock == false)
                {
                    MessageBox.Show("You do not own that stock");
                }
                else if (hasAmount == false)
                {
                    MessageBox.Show("You do not own enough of that stock you are trying to sell");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Number of shares not specified");
            }
        }
        /*
        *  FUNCTION       : updateTransactions()
        *  DISCRIPTION    : updates the transactions tab lists when the user buys or sells
        *  PARAMETER      : none
        *  RETURN         : none
        */
        private void updateTransactions()
        {
            investor_Transaction.Items.Clear();
            transaction_TypeList.Items.Clear();
            transaction_DateList.Items.Clear();
            investor_Transaction.Items.AddRange(TransactionAmountsHistory.ToArray());
            transaction_TypeList.Items.AddRange(TransactionTypesHistory.ToArray());
            transaction_DateList.Items.AddRange(TransactionDatesHistory.ToArray());
        }
        /*
        *  FUNCTION       : checkForStock()
        *  DISCRIPTION    : confirms whether the user has a stock in their profile
        *  PARAMETER      : none
        *  RETURN         : bool hasStock - whether the user has purchased the stock or not
        */
        private bool checkForStock()
        {
            bool hasStock = false;

            //Check that the user has stocks from the selected company
            if (CompanySymbols.Contains(selectedSymbol))
            {
                hasStock = true;
            }

            return hasStock;
        }
        /*
        *  FUNCTION       : checkForStockAmount()
        *  DISCRIPTION    : checks if the user has the amount of stocks trying to be sold
        *  PARAMETER      : int sellAmount - the amount of stocks to be sold
        *  RETURN         : bool hasAmount - whether the user has enough stocks to sell
        */
        private bool checkForStockAmount(int sellAmount)
        {
            bool hasAmount = false;

            //Get the index for the purchased amount of stocks from the position of the symbol in the companySymbols list
            int index = CompanySymbols.IndexOf(selectedSymbol);

            if (sellAmount <= StocksOwned[index])
            {
                hasAmount = true;
            }

            return hasAmount;
        }

        /*
         *  FUNCTION       : userProfileExtract()
         *  DISCRIPTION    : Extracting user data from string array
         *  PARAMETER      : none
         *  RETURN         : None
         */

        public void userProfileExtract()
        {
            char delimiter = ',';
            int counter = 0;
            string[] substrings = userData.Split(delimiter);

            foreach (var substring in substrings)
            {
                if (counter == 0) { userName= substring; }
                if (counter == 1) { company = substring; }
                if (counter == 2) { stockAmount = substring; }
                if (counter == 3) { transaction = substring; }
                if (counter == 4) { transactionDate = substring; }
                if (counter == 5) { transactionType = substring; }
                if (counter == 6) { bankBalance = substring; }
                if (counter == 7) { role = substring; }
                counter++;
            }
        }

        /*
        *  FUNCTION       : loadTransactions()
        *  DISCRIPTION    : parses user profile information into appropriate lists and displays information
        *  PARAMETER      : 
        *  RETURN         : none
        */
        private void loadTransactions()
        {
            if (company != "")
            {
                CompanySymbolsHistory.Add(company);
                CompanySymbols.Add(company);
            }
            if (stockAmount != "")
            {
                stockTranacrtionHistory.Add(Convert.ToInt32(stockAmount));
                StocksOwned.Add(Convert.ToInt32(stockAmount));
            }
            if (transaction != "")
            {
                TransactionAmountsHistory.Add(transaction);
            }
            if (transactionDate != "")
            {
                TransactionDatesHistory.Add(transactionDate);
            }
            if (transactionType != "")
            {
                TransactionTypesHistory.Add(transactionType);
            }

            //Diplsay bank balance
            if (bankBalance != "")
            {
                userBankBalance = Convert.ToDouble(bankBalance);
                bankBalanceTB.Items[0] = userBankBalance.ToString();
            }

            updateTransactions();
        }

        /*
        *  FUNCTION       : toolStripMenuItem1_Click()
        *  DISCRIPTION    : saves the user profile back to the server/database
        *  PARAMETER      : 
        *  RETURN         : none
        */
        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            string sendInfo = userName + "," + CompanySymbolsHistory[CompanySymbols.Count - 1] + "," +
                 stockTranacrtionHistory[stockTranacrtionHistory.Count - 1].ToString() + "," + TransactionAmountsHistory[TransactionAmountsHistory.Count - 1] + "," +
                 TransactionTypesHistory[TransactionTypesHistory.Count - 1] + "," + TransactionDatesHistory[TransactionDatesHistory.Count - 1] + ","
                 + userBankBalance + ","+ role;

            UdpClient udpClient = new UdpClient();
            udpClient.Connect("104.234.44.183", 8000);
            Byte[] saveData = Encoding.ASCII.GetBytes(sendInfo);
            udpClient.Send(saveData, saveData.Length);
        }
    }
}