﻿/*
*	FILE			: ESMIS_GET_STOCK.cs
*	PROJECT			: ESMIS
*	PROGRAMMER		: Posted on January 16, 2015 by Rod Stephens http://csharphelper.com/blog/2015/01/get-stock-prices-from-the-internet-in-c/	
*	MODIFIED		: BRIAN HINDS, JAMES SIMPSOM, JACKSON RUBY
*	MODIFIED VERSION: 1.2.0
*	DESCRIPTION 	: This Class gets the stock information for client
*/
using System;
using System.IO;
using System.Net;
using System.Windows.Forms;

namespace ESMIS
{
    internal class ESMIS_GET_STOCK
    {
        private string stock_url = "";
        private const string stock_base_url = "http://download.finance.yahoo.com/d/quotes.csv?s=@&f=d1ohgpv";
        private const string stock_range_url = "http://download.finance.yahoo.com/d/quotes.csv?s=@&f=jkam3m4s";
        private string quoteResults;
        private string stock_results;
        /*
       *  FUNCTION       : getStockQuote
       *  DISCRIPTION    : get the stock quotes
       *  PARAMETER      : (string symbol)
       *  RETURN         : stock_results
       */

        public string getStockQuote(string symbol)
        {
            stock_url = symbol;
            if (stock_url != "")
            {
                // Perpend the base URL.
                stock_url = stock_base_url.Replace("@", stock_url);
                // Get the response.
                try
                {
                    // Get the web response.
                    quoteResults = GetStockResponse(stock_url);
                    Console.WriteLine(quoteResults.Replace("\\r\\n", "\r\n"));
                    // Pull out the current prices.
                    string[] line = quoteResults.Split(new char[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);
                    stock_results = line[0].ToString();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Read Error",
                       MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            return stock_results;
        }

        /*
        *  FUNCTION       : getStockAverage
        *  DISCRIPTION    : get the stock quotes
        *  PARAMETER      : (string symbol)
        *  RETURN         : stock_results
        */

        public string getStockRange(string symbol)
        {
            stock_url = symbol;
            if (stock_url != "")
            {
                // Perpend the base URL.
                stock_url = stock_range_url.Replace("@", stock_url);
                // Get the response.
                try
                {
                    // Get the web response.
                    quoteResults = GetStockResponse(stock_url);
                    Console.WriteLine(quoteResults.Replace("\\r\\n", "\r\n"));
                    // Pull out the current prices.
                    string[] line = quoteResults.Split(new char[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);
                    stock_results = line[0].ToString();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Read Error",
                       MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            return stock_results;
        }

        /*
       *  FUNCTION       : GetStockResponse
       *  DISCRIPTION    : Get a web response
       *  PARAMETER      : (string url)
       *  RETURN         : results
       */

        private string GetStockResponse(string url)
        {
            // Make a WebClient.
            WebClient stock_client = new WebClient();
            // Get the indicated URL.
            Stream response = stock_client.OpenRead(url);
            // Read the result.
            using (StreamReader in_stream = new StreamReader(response))
            {
                // Get the results.
                string results = in_stream.ReadToEnd();
                // Close the stream reader and its underlying stream.
                in_stream.Close();
                // Return the result.
                return results;
            }
        }
    }
}