﻿/*
*	FILE			: ESMIS_HASH.cs
*	PROJECT			: ESMIS
*	PROGRAMMER		: by Richard Carr, published at http://www.blackwasp.co.uk/SaltedPasswordHashing.aspx
*	FIRST VERSION	: 0.0.0
*	DESCRIPTION 	: Create hash from salt and password
*/
using System;
using System.Security.Cryptography;
using System.Text;

namespace ESMIS
{
    internal class ESMIS_HASH
    {
        /*
      *  FUNCTION       : GenerateSalt()
      *  DISCRIPTION    : Generates salt value
      *  PARAMETER      : None
      *  RETURN         : string
      */

        public string GenerateSalt()
        {
            return Guid.NewGuid().ToString("N");
        }

        /*
       *  FUNCTION       : HashPassword
       *  DISCRIPTION    : Hash user password
       *  PARAMETER      : string password, string salt
       *  RETURN         : string
       */

        public string HashPassword(string password, string salt)
        {
            string combined = password + salt;
            return HashString(combined);
        }

        /*
       *  FUNCTION       : CheckPassword
       *  DISCRIPTION    : Checks the password with salt+hash from server
       *  PARAMETER      : string password, string salt, string hash
       *  RETURN         : bool
       */

        public bool CheckPassword(string password, string salt, string hash)
        {
            return HashPassword(password, salt) == hash;
        }

        /*
       *  FUNCTION       : HashString
       *  DISCRIPTION    : Hash string that does the hashing
       *  PARAMETER      : string toHash
       *  RETURN         : string
       */

        private string HashString(string toHash)
        {
            using (SHA512CryptoServiceProvider sha = new SHA512CryptoServiceProvider())
            {
                byte[] dataToHash = Encoding.UTF8.GetBytes(toHash);
                byte[] hashed = sha.ComputeHash(dataToHash);
                return Convert.ToBase64String(hashed);
            }
        }
    }
}