﻿/*
*	FILE			: ESMIS_LOGIN
*	PROJECT			: ESMIS
*	PROGRAMMER		: BRIAN HINDS, JAMES SIMPSOM, JACKSON RUBY
*	FIRST VERSION	: 2.1.0
*	DESCRIPTION 	: Main login, registration, and password reset form for user
*/

using System;
using System.ComponentModel;
using System.Configuration;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Windows.Forms;

namespace ESMIS
{
    public partial class ESMIS_LOGIN : Form
    {
        /// <summary>
        /// Variables that stores parameters form WAN connection
        /// </summary>
        private TcpClient client;

        public StreamReader ESMIS_STR;
        public StreamWriter ESMIS_STW;
        public string data_Send;
        public string data_receive;
        private string local_IP;
        private string _salt = "";
        private string _hash;
        private int wrongEntry = 0;
        public static string userProfile;

        /// <summary>
        /// Instantiation for the required classes
        /// </summary>
        private ESMIN_ENCRYPTION _Encrypt = new ESMIN_ENCRYPTION();

        private ESMIS_PASSWORD _Hash = new ESMIS_PASSWORD();
        private ESMIS_HASH _HashPW = new ESMIS_HASH();
        private ESMIS_LOG logUser = new ESMIS_LOG();
        private USER_REGISTRATION _Register = new USER_REGISTRATION();
        private string _ServerIP_Connection = ConfigurationManager.ConnectionStrings["ESMIS_SERVER_IP"].ConnectionString;

        /// <summary>
        /// Protocol tags that directs data flow
        /// </summary>
        private string _ProtocolA = "USERLOGIN";

        private string _ProtocolB = "USERSALT";
        private string _ProtocolC = "USERHASH";
        private string _ProtocolD = "USERREGISTER";
        private string _ProtocolE = "USERREGISTERPASS";
        private string _ProtocolF = "USERREGISTERFAIL";
        private string _ProtocolG = "USERPASSWORDRESET";
        private string _ProtocolH = "USERRESETVALIDATIONPASS";
        private string _ProtocolHH = "USERRESETVALIDATIONFAIL";
        private string _ProtocolJ = "USERNEWPASSWORD";
        private string _ProtocolK = "USERPASSWORDRESET";
        private string _ProtocolL = "USERPASSWORDRESETFAIL";
        private bool regStatus = true;

        /// <summary>
        /// Variables for password reset parameters
        /// </summary>

        private string studentID;
        private bool test = true;
        private string secretQA;
        private string secretQB;
        private string secretAA;
        private string secretAB;
        private bool esmis_exist = false;
        /// <summary>
        /// Constructor for the login form
        /// </summary>
        ///

        public ESMIS_LOGIN()
        {
            InitializeComponent();
            password_BoxA.UseSystemPasswordChar = true;
            password_BoxB.UseSystemPasswordChar = true;
            passwordA_box.UseSystemPasswordChar = true;
            passwordB_box.UseSystemPasswordChar = true;
            submit_Pass.Enabled = false;
            passwordA_box.Enabled = false;
            passwordB_box.Enabled = false;
        }

        /*
        *  FUNCTION       : ESMIS_LOGIN_Load
        *  DESCRIPTION   : Connect to server
        *  PARAMETER      : (object sender, EventArgs e)
        *  RETURN         : None
        */

        private void ESMIS_LOGIN_Load(object sender, EventArgs e)
        {
            getLocalIp();
            connect_to_server();
            _Encrypt.getKey();
        }

        /*
        *  FUNCTION       : connect_to_server()
        *  DESCRIPTION   : Connect to defined Ip and Port
        *  PARAMETER      : None
        *  RETURN         : None
        */

        private void connect_to_server()
        {
            client = new TcpClient();
            IPEndPoint IP_End = new IPEndPoint(IPAddress.Parse(_ServerIP_Connection), int.Parse("8080"));

            try
            {
                client.Connect(IP_End);
                if (client.Connected)
                {
                    serverConnection.Image = Properties.Resources.tick;
                    ESMIS_STR = new StreamReader(client.GetStream());
                    ESMIS_STW = new StreamWriter(client.GetStream());
                    ESMIS_STW.AutoFlush = true;

                    backgroundWorker1.RunWorkerAsync(); // Start receiving data in background
                    backgroundWorker2.WorkerSupportsCancellation = true; // Ability to cancel this thread.
                    logUser.user_log("Connection Successful");
                }
            }
            catch (Exception x)
            {
                MessageBox.Show(x.Message.ToString());
                serverConnection.Image = Properties.Resources.Erase;
                logUser.user_log("Connection Fail-->" + x.Message.ToString());
            }
        }

        /*
        *  FUNCTION       : loginBtn_Click
        *  DESCRIPTION   : Login button
        *  PARAMETER      : (object sender, EventArgs e)
        *  RETURN         : None
        */

        private void loginBtn_Click(object sender, EventArgs e)
        {
            if (password_BoxA.Text != "" & userName_Box.Text != "")
            {
                string rawData;

                // Combining data for encryption
                rawData =
                local_IP + "," +
                userName_Box.Text + "," +
                _ProtocolA;

                // Send encrypt data to server
                sendClientData(rawData);
            }
            if (string.IsNullOrEmpty(password_BoxA.Text) || string.IsNullOrEmpty(userName_Box.Text))
            {
                logUser.user_log("User left credential field empty");
                wrongEntry++;
                MessageBox.Show("Check userID or Password:", "Input Box Empty", MessageBoxButtons.OK, MessageBoxIcon.Error);
                if (wrongEntry >= 3)
                {
                    logUser.user_log("Application restrict access due to multiple attempts");
                    password_BoxA.Enabled = false;
                    userName_Box.Enabled = false;
                    loginBtn.Enabled = false;
                    MessageBox.Show("Please reset your password", "Multiple Tries", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /*
        *  FUNCTION       : backgroundWorker1_DoWork_1
        *  DESCRIPTION   : Receive data from server
        *  PARAMETER      : (object sender, DoWorkEventArgs e)
        *  RETURN         : None
        */

        private void backgroundWorker1_DoWork_1(object sender, DoWorkEventArgs e)
        {
            string _dcrpData;
            while (client.Connected)
            {
                try
                {
                    //Get server data
                    data_receive = ESMIS_STR.ReadLine();
                    //Decrypt server data
                    _dcrpData = _Encrypt.Decrypt(data_receive, _Encrypt.key);
                    //Determine what to do with data
                    clientAction(_dcrpData);
                }
                catch (Exception x)
                {
                    MessageBox.Show(x.Message.ToString());
                    logUser.user_log("Fail to receive data from server");
                }
            }
        }

        /*
        *  FUNCTION       : backgroundWorker2_DoWork_1
        *  DESCRIPTION   : Send data to server
        *  PARAMETER      : (object sender, DoWorkEventArgs e)
        *  RETURN         : None
        */

        private void backgroundWorker2_DoWork_1(object sender, DoWorkEventArgs e)
        {
            if (client.Connected)
            {
                ESMIS_STW.WriteLine(data_Send);
                logUser.user_log("Data sent to server");
            }
            else
            {
                MessageBox.Show("Send Fail!");
                logUser.user_log("Fail to send data to server");
            }
            backgroundWorker2.CancelAsync();
        }

        /*
        *  FUNCTION       : getLocalIp()
        *  DESCRIPTION   : Get local IP
        *  PARAMETER      : None
        *  RETURN         : None
        */

        private void getLocalIp()
        {
            IPAddress[] localIP = Dns.GetHostAddresses(Dns.GetHostName());
            foreach (IPAddress address in localIP)
            {
                if (address.AddressFamily == AddressFamily.InterNetwork)
                {
                    local_IP = address.ToString();
                }
            }
        }

        /*
        *  FUNCTION       : cancelBtn_Click
        *  DESCRIPTION   : Close application
        *  PARAMETER      : (object sender, EventArgs e)
        *  RETURN         : None
        */

        private void cancelBtn_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        /*
        *  FUNCTION       : sendClientData
        *  DISCRIPTION    : Sends data to the server
        *  PARAMETER      : string encData
        *  RETURN         : None
        */

        private void sendClientData(string rawData)
        {
            // Send encrypt data to server
            data_Send = _Encrypt.Encrypt(rawData, _Encrypt.key);
            backgroundWorker2.RunWorkerAsync();
        }

        /*
        *  FUNCTION       : userReg()
        *  DISCRIPTION    : Validates user registration form
        *  PARAMETER      : None
        *  RETURN         : None
        */

        private bool userReg()
        {
            regStatus = true;
            if (string.IsNullOrEmpty(firstName_Box.Text))
            { MessageBox.Show("Invalid Name"); regStatus = false; }
            if (string.IsNullOrEmpty(lastName_Box.Text))
            { MessageBox.Show("Invalid Name"); regStatus = false; }
            if (string.IsNullOrEmpty(studentID_Box.Text))
            { MessageBox.Show("Invalid Student ID"); regStatus = false; }
            if (string.IsNullOrEmpty(password_BoxB.Text))
            { MessageBox.Show("Invalid Password"); regStatus = false; }
            if (string.IsNullOrEmpty(instructorCode_Box.Text))
            { MessageBox.Show("Invalid Instructor Code"); regStatus = false; }
            if (string.IsNullOrEmpty(courseCode_Box.Text))
            { MessageBox.Show("Invalid Course Code"); regStatus = false; }
            if (string.IsNullOrEmpty(institute_box.Text))
            { MessageBox.Show("Invalid Institution"); regStatus = false; }
            if (string.IsNullOrEmpty(email_BoxA.Text))
            { MessageBox.Show("Invalid Email"); regStatus = false; }
            if (string.IsNullOrEmpty(email_BoxB.Text))
            { MessageBox.Show("Invalid Email"); regStatus = false; }
            if (email_BoxA.Text != email_BoxB.Text)
            { MessageBox.Show("Email Mismatch"); regStatus = false; }
            if (securityQuestionA.SelectedItem == null || securityQuestionB.SelectedItem == null)
            { MessageBox.Show("Secret Question Required"); regStatus = false; }
            if (string.IsNullOrEmpty(securityAnswerA.Text))
            { MessageBox.Show("Security Answer Needed"); regStatus = false; }
            if (string.IsNullOrEmpty(securityAnswerB.Text))
            { MessageBox.Show("Security Answer Needed"); regStatus = false; }

            return regStatus;
        }

        /*
        *  FUNCTION       : submit_Button_Click
        *  DISCRIPTION    : Submit registration Data
        *  PARAMETER      : (object sender, EventArgs e)
        *  RETURN         : None
        */

        private void submit_Button_Click(object sender, EventArgs e)
        {
            bool chk;
            string _salt = _Hash.creatSalt();
            string _hash = _Hash.hash_salt(password_BoxB.Text);
            string rawData;

            //Validate form fields
            chk = userReg();

            //Combine field data is all checks out
            if (chk == true)
            {
                rawData = _Register.getRegData(firstName_Box.Text, lastName_Box.Text, studentID_Box.Text, instructorCode_Box.Text, courseCode_Box.Text, institute_box.Text, email_BoxB.Text, securityQuestionA.SelectedItem.ToString(), securityQuestionB.SelectedItem.ToString(), securityAnswerA.Text, securityAnswerB.Text) + "," + _salt + "," + _hash + "," + _ProtocolD;

                //Sending data to the server.
                sendClientData(rawData);
            }
        }

        /*
        *  FUNCTION       : clientAction
        *  DISCRIPTION    : This function takes certain action base on protocols
        *  PARAMETER      : (string serverData)
        *  RETURN         : None
        */

        private void clientAction(string serverData)
        {
            string reply_to_server;
            //Checking server results for confirmation

            if (serverData.Contains(_ProtocolB))
            {
                if (serverData.Contains("InvalidUser"))
                {
                    MessageBox.Show("User Do Not Exist", "Login Failed", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else
                {
                    //Process the password to create a hash value.
                    reply_to_server = processHash(serverData);
                    //Return hash to the server to be checked.
                    sendClientData(reply_to_server + "," + _ProtocolC);
                }
            }

            //Successful login, user profile is downloaded.
            if (serverData.Contains("Client"))
            {
                userProfile = serverData;
                CloseForm();
                ESMIS_CLIENT clientForm = new ESMIS_CLIENT();
                clientForm.ShowDialog();
            }
            //Successful login, user profile is downloaded.
            if (serverData.Contains("Admin"))
            {
                userProfile = serverData;
                CloseForm();
                ESMIS_ADMIN adminForm = new ESMIS_ADMIN();
                adminForm.ShowDialog();
            }

            //Registration fail
            if (serverData == _ProtocolF)
            {
                MessageBox.Show("User Already Exist", "Registration Failed", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                logUser.user_log("Registration Failed");
            }

            //Registration success
            if (serverData == _ProtocolE)
            {
                MessageBox.Show("You may login", "Registration Successful", MessageBoxButtons.OK, MessageBoxIcon.Information);
                logUser.user_log("Registration Successful");
            }
            //User's reset data checks out
            if (serverData == _ProtocolH)
            {
                submit_Pass.Enabled = true;
                passwordA_box.Enabled = true;
                passwordB_box.Enabled = true;
                studentID_BoxA.Enabled = false;
                test = true;
                reset_icon.Image = Properties.Resources.tick;
            }
            //Information entered do not match database records
            if (serverData == _ProtocolHH)
            {
                reset_icon.Image = Properties.Resources.Erase;
                MessageBox.Show("Process unsuccessful\nPlease check you entered the correct information", "Reset Failed", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            //Server fail to reset password
            if (serverData == _ProtocolL)
            {
                password_icon.Image = Properties.Resources.Erase;
                MessageBox.Show("Process unsuccessful\nUnable to reset your password at this time", "Reset Failed", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            //Password reset process successful
            if (serverData == _ProtocolK)
            {
                password_icon.Image = Properties.Resources.tick;
                MessageBox.Show("Process Successful\nYou may now login", "Reset Confirmed", MessageBoxButtons.OK, MessageBoxIcon.Information);

                passwordA_box.Text = "";
                passwordB_box.Text = "";
                studentID_BoxA.Text = "";
                SA_BoxA.Text = "";
                SA_BoxB.Text = "";
            }
        }

        /*
        *  FUNCTION       : processHash
        *  DISCRIPTION    : Generate a hash value for server to check valid user.
        *  PARAMETER      : (string serverData)
        *  RETURN         : None
        */

        private string processHash(string serverData)
        {
            char delimiter = ',';
            int count = 0;
            //Create a string array
            string[] substrings = serverData.Split(delimiter);
            //Extract salt value for user
            foreach (var substring in substrings)
            {
                if (count == 0) { _salt = substring; count++; }
            }
            _hash = _HashPW.HashPassword(password_BoxA.Text, _salt);
            return _hash;
        }

        /*
      *  FUNCTION       : reset_information()
      *  DISCRIPTION    : Extract form information after testing
      *  PARAMETER      : None
      *  RETURN         : None
      */

        private void reset_information()
        {
            esmis_security_questions();

            test = true;

            if (!string.IsNullOrEmpty(studentID_BoxA.Text) & test == true) { studentID = studentID_BoxA.Text + ","; test = true; }
            else { test = false; }
            if (!string.IsNullOrEmpty(secretQA) & test == true) { test = true; }
            else { test = false; }
            if (!string.IsNullOrEmpty(secretQB) & test == true) { test = true; }
            else { test = false; }
            if (!string.IsNullOrEmpty(secretAA) & test == true) { test = true; }
            else { test = false; }
            if (!string.IsNullOrEmpty(secretAB) & test == true) { test = true; }
            else { test = false; }
            if (test == false) { MessageBox.Show("Please check the form fields.", "Empty Field", MessageBoxButtons.OK, MessageBoxIcon.Error); }
        }

        /*
            *  FUNCTION       : esmis_security_questions
            *  DISCRIPTION    : extract security questions and answers
            *  PARAMETER      : None
            *  RETURN         : None
            */

        private void esmis_security_questions()
        {
            if (SQ_BoxA.SelectedItem == null || SQ_BoxB.SelectedItem == null)
            { MessageBox.Show("Please check select a secret question.", "Empty Question", MessageBoxButtons.OK, MessageBoxIcon.Error); }
            else
            {
                secretQA = SQ_BoxA.SelectedItem.ToString() + ",";
                secretQB = SQ_BoxB.SelectedItem.ToString() + ",";
                secretAA = SA_BoxA.Text + ",";
                secretAB = SA_BoxB.Text + ',';
            }
        }

        /*
      *  FUNCTION       : reset_Button_Click
      *  DISCRIPTION    : Submits user reset information
      *  PARAMETER      : (object sender, EventArgs e)
      *  RETURN         : None
      */

        private void reset_Button_Click(object sender, EventArgs e)
        {
            reset_information();

            if (test)
            {
                sendClientData(studentID + secretQA + secretQB + secretAA + secretAB + _ProtocolG);
            }
        }

        /*
        *  FUNCTION       : submit_Pass_Clic
        *  DISCRIPTION    : Submit new has and salt for password reset
        *  PARAMETER      : (object sender, EventArgs e)
        *  RETURN         : None
        */

        private void submit_Pass_Click(object sender, EventArgs e)
        {
            test = true;
            if (passwordA_box.Text == passwordB_box.Text & test == true)
            {
                if (!string.IsNullOrEmpty(passwordA_box.Text) || !string.IsNullOrEmpty(passwordB_box.Text) & test == true)
                {
                    string _salt = _Hash.creatSalt();
                    string _hash = _Hash.hash_salt(passwordB_box.Text);

                    sendClientData(studentID_BoxA.Text + "," + _salt + "," + _hash + "," + _ProtocolJ);
                }
                else { test = false; }
            }
            else { test = false; }

            if (test == false) { MessageBox.Show("Process unsuccessful", "reset Failed", MessageBoxButtons.OK, MessageBoxIcon.Error); }
        }

        /*
        *  FUNCTION       : CloseForm
        *  DISCRIPTION    : Correctly launch client form after successful login
        *  PARAMETER      : None
        *  RETURN         : None
        */

        private delegate void BlankDelegate();

        private void CloseForm()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new BlankDelegate(this.CloseForm));
            }
            else
            {
                this.Hide();
            }
        }
    }
}