﻿/*
*	FILE			: ESMIS_DATABASE.cs
*	PROJECT			: ESMIS
*	PROGRAMMER		: BRIAN HINDS, JAMES SIMPSOM, JACKSON RUBY
*	FIRST VERSION	: 2.0.0
*	DESCRIPTION 	: This file handles the SQL commands for the database
*/
using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Windows.Forms;

namespace ESMIS_SERVER
{
    internal class ESMIS_DATABASE
    {
        private const string DefaultBankAmount = "10000";
        private string _salt = "InvalidUser";
        private string _results;
        private string _DataBase_Connection = ConfigurationManager.ConnectionStrings["ESMIS_db_CONNECTION"].ConnectionString;
        /*
        *  FUNCTION       : checkLogin
        *  DISCRIPTION    : Checks if the user logging in exist
        *  PARAMETER      : (string _user)
        *  RETURN         : string Salt value if user exist.
        */

        public string checkLogin(string _user)
        {
            SqlConnection _connectDB = new SqlConnection(_DataBase_Connection);
            SqlDataAdapter sda = new SqlDataAdapter("Select Salt from USER_TABLE where UserName='" + _user + "'", _connectDB);
            DataTable dt = new DataTable();
            sda.Fill(dt);
            if (dt.Rows.Count == 1)
            {
                _salt = dt.Rows[0][0].ToString();
            }
            return _salt;
        }

        /*
        *  FUNCTION       : checkHash
        *  DISCRIPTION    : Checking user hash with their salt.
        *  PARAMETER      : (string _hash)
        *  RETURN         : String array of user data.
        */

        public string checkHash(string _hash)
        {
            string userName = "";
            string company = "";
            string stockAmount = "";
            string transaction = "";
            string transactionType = "";
            string transactionDate = "";
            string bankBalance = "";
            string role = "";
            string combinedData = "";

            SqlConnection _connectDB = new SqlConnection(_DataBase_Connection);
            SqlDataAdapter sda = new SqlDataAdapter("Select Hash from USER_TABLE where Hash='" + _hash + "'", _connectDB);
            DataTable dt = new DataTable();
            sda.Fill(dt);
            if (dt.Rows.Count == 1)
            {
                SqlConnection _connectDA = new SqlConnection(_DataBase_Connection);
                SqlDataAdapter sdaA = new SqlDataAdapter("Select * from USER_PROFILE where Hash='" + _hash + "'", _connectDA);
                DataTable dtA = new DataTable();
                sdaA.Fill(dtA);
                foreach (DataRow dtr in dtA.Rows)
                {
                    userName = dtr["UserName"].ToString();
                    company = dtr["Companies"].ToString();
                    stockAmount = dtr["StockAmount"].ToString();
                    transaction = dtr["Transaction"].ToString();
                    transactionDate = dtr["TransactionDate"].ToString();
                    transactionType = dtr["transactionType"].ToString();
                    bankBalance = dtr["BankBalance"].ToString();
                }
                _connectDA.Close();

                SqlConnection _connectDC = new SqlConnection(_DataBase_Connection);
                SqlDataAdapter sdaC = new SqlDataAdapter("Select Role from USER_TABLE where Hash='" + _hash + "'", _connectDC);
                DataTable dtC = new DataTable();
                sdaC.Fill(dtC);

                if (dtC.Rows.Count == 1)
                {
                    role = dtC.Rows[0][0].ToString();
                }
            }

            combinedData = userName + "," + company + "," + stockAmount + "," + transaction + "," + transactionDate + "," + transactionType + "," + bankBalance + "," + role;

            return combinedData;
        }

        /*
        *  FUNCTION       : checkNewUser
        *  DISCRIPTION    : Check if registered user exists
        *  PARAMETER      : string _email
        *  RETURN         : bool
        */

        public bool checkNewUser(string _user)
        {
            bool results = false;
            SqlConnection _connectDB = new SqlConnection(_DataBase_Connection);
            SqlDataAdapter sda = new SqlDataAdapter("Select Email from USER_TABLE where UserName='" + _user + "'", _connectDB);
            DataTable dt = new DataTable();
            sda.Fill(dt);
            if (dt.Rows.Count == 1)
            {
                results = true;
            }
            else
            {
                results = false;
            }
            return results;
        }

        /*
       *  FUNCTION       : _AddUser
       *  DISCRIPTION    : Adding new users to the data base
       *  PARAMETER      : (string A, string B, string C, string D, string E, string F, string G, string H, string I, string J, string K, string L, string M, string N)
       *  RETURN         : None
       */

        public void _AddUser(string A, string B, string C, string D, string E, string F, string G, string H, string I, string J, string K, string L, string M, string N)
        {
            SqlConnection _connectDB = new SqlConnection(_DataBase_Connection);
            SqlCommand nData = new SqlCommand("Insert into USER_TABLE (UserName, Salt, Hash, Role, FirstName, LastName, StudentID, InstructorCode, CourseCode, InstitutionCode, Email, SecretQA, SecretQB, SecretAA, SecretAB) Values(@UserName, @Salt, @Hash, @Role, @FirstName, @LastName, @StudentID, @InstructorCode, @CourseCode, @InstitutionCode, @Email, @SecretQA, @SecretQB, @SecretAA, @SecretAB)", _connectDB);
            nData.Parameters.AddWithValue("@UserName", A);
            nData.Parameters.AddWithValue("@Salt", B);
            nData.Parameters.AddWithValue("@Hash", C);
            nData.Parameters.AddWithValue("@Role", "Client");
            nData.Parameters.AddWithValue("@FirstName", D);
            nData.Parameters.AddWithValue("@LastName", E);
            nData.Parameters.AddWithValue("@StudentID", F);
            nData.Parameters.AddWithValue("@InstructorCode", G);
            nData.Parameters.AddWithValue("@CourseCode", I);
            nData.Parameters.AddWithValue("@InstitutionCode", H);
            nData.Parameters.AddWithValue("@Email", J);
            nData.Parameters.AddWithValue("@SecretQA", K);
            nData.Parameters.AddWithValue("@SecretQB", L);
            nData.Parameters.AddWithValue("@SecretAA", M);
            nData.Parameters.AddWithValue("@SecretAB", N);

            _connectDB.Open();
            nData.ExecuteNonQuery();
            _connectDB.Close();

            SqlConnection _connectDA = new SqlConnection(_DataBase_Connection);
            SqlCommand MData = new SqlCommand("Insert into USER_PROFILE (UserName, Hash, StudentID, BankBalance) Values(@userName, @hash, @studentid, @bankBalance)", _connectDA);
            MData.Parameters.AddWithValue("@userName", A);
            MData.Parameters.AddWithValue("@hash", C);
            MData.Parameters.AddWithValue("@studentid", F);
            MData.Parameters.AddWithValue("@bankBalance", DefaultBankAmount);

            _connectDA.Open();
            MData.ExecuteNonQuery();
            _connectDA.Close();
        }

        /*
        *  FUNCTION       : checkUserChangeRequest
        *  DISCRIPTION    : Checking if user security questions pass
        *  PARAMETER      : (string StudentID, string SecretQA, string SecretQB, string SecretAA, string SecretAB)
        *  RETURN         : bool
        */

        public bool checkUserChangeRequest(string StudentID, string SecretQA, string SecretQB, string SecretAA, string SecretAB)
        {
            bool chkout = false;
            // Check database for user and first secret question
            SqlConnection _connectDB = new SqlConnection(_DataBase_Connection);
            SqlDataAdapter sda = new SqlDataAdapter("Select SecretQA from USER_TABLE where StudentID='" + StudentID + "'", _connectDB);
            DataTable dt = new DataTable();
            sda.Fill(dt);
            if (dt.Rows.Count == 1)
            {
                _results = dt.Rows[0][0].ToString();
                chkout = true;
            }
            // If Question is a match then check first answer
            if (SecretQA == _results & chkout)
            {
                SqlConnection _connectDBA = new SqlConnection(_DataBase_Connection);
                SqlDataAdapter sdaA = new SqlDataAdapter("Select SecretAA from USER_TABLE where StudentID='" + StudentID + "'", _connectDBA);
                DataTable dtA = new DataTable();
                sdaA.Fill(dtA);
                if (dtA.Rows.Count == 1)
                {
                    _results = dtA.Rows[0][0].ToString();
                    chkout = true;
                }
                else
                { chkout = false; }
            }
            // If first test past then continue to second question
            if (SecretAA == _results & chkout)
            {
                SqlConnection _connectDBB = new SqlConnection(_DataBase_Connection);
                SqlDataAdapter sdaB = new SqlDataAdapter("Select SecretQB from USER_TABLE where StudentID='" + StudentID + "'", _connectDBB);
                DataTable dtB = new DataTable();
                sdaB.Fill(dtB);
                if (dtB.Rows.Count == 1)
                {
                    _results = dtB.Rows[0][0].ToString();
                    chkout = true;
                }
            }
            // Checking database for second answer for second question
            if (SecretQB == _results & chkout)
            {
                SqlConnection _connectDBC = new SqlConnection(_DataBase_Connection);
                SqlDataAdapter sdaC = new SqlDataAdapter("Select SecretAB from USER_TABLE where StudentID='" + StudentID + "'", _connectDBC);
                DataTable dtC = new DataTable();
                sdaC.Fill(dtC);
                if (dtC.Rows.Count == 1)
                {
                    _results = dtC.Rows[0][0].ToString();
                    chkout = true;
                }
                else
                { chkout = false; }
            }
            // If second answer checks out then will allow user to reset password
            if (_results == SecretAB & chkout)
            { chkout = true; }
            else { chkout = false; }

            return chkout;
        }

        /*
        *  FUNCTION       : updateUserPasswordData
        *  DISCRIPTION    : Update user new hash and salt
        *  PARAMETER      : (string StudentID, string _hash, string _salt)
        *  RETURN         : bool
        */

        public bool updateUserPasswordData(string StudentID, string _hash, string _salt)
        {
            bool results;
            try
            {
                //Update user table
                SqlConnection _connectDB = new SqlConnection(_DataBase_Connection);
                SqlCommand sda = new SqlCommand("UPDATE USER_TABLE SET Hash=@HashValue, Salt=@SaltValue where StudentID=@StudentIDValue", _connectDB);
                sda.Parameters.AddWithValue("@HashValue", _hash);
                sda.Parameters.AddWithValue("@SaltValue", _salt);
                sda.Parameters.AddWithValue("@StudentIDValue", StudentID);
                _connectDB.Open();
                sda.ExecuteNonQuery();
                _connectDB.Close();

                //Update profile table
                SqlConnection _connectDA = new SqlConnection(_DataBase_Connection);
                SqlCommand sdaA = new SqlCommand("UPDATE USER_PROFILE SET Hash=@HashValue where StudentID=@StudentIDValue", _connectDA);
                sdaA.Parameters.AddWithValue("@HashValue", _hash);
                sdaA.Parameters.AddWithValue("@StudentIDValue", StudentID);
                _connectDA.Open();
                sdaA.ExecuteNonQuery();
                _connectDA.Close();

                results = true;
            }
            catch
            {
                results = false;
            }

            return results;
        }
        /*
        *  FUNCTION       : saveGameSession
        *  DISCRIPTION    : Save last game session
        *  PARAMETER      : (string userName, string company, string stockAmount, string transaction, string transactionDate, string transactionType, string bankAmount)
        *  RETURN         : None
        */
        public void saveGameSession(string userName, string company, string stockAmount, string transaction, string transactionDate, string transactionType, string bankAmount)
        {

            try
            {
                //Update profile table
                SqlConnection _connectDA = new SqlConnection(_DataBase_Connection);
                SqlCommand sda = new SqlCommand("UPDATE USER_PROFILE SET Companies=@company,  TransactionType=@transactionType, TransactionDate=@transactionDate, BankBalance=@bankAmount, StockAmount=@stockAmount where UserName=@UserName", _connectDA);
                sda.Parameters.AddWithValue("@UserName", userName);
                sda.Parameters.AddWithValue("@company", company);
                //sda.Parameters.AddWithValue("@transaction", transaction); --> (0x80131904) incorrect syntax near keyword Transaction
                sda.Parameters.AddWithValue("@transactionType", transactionType);
                sda.Parameters.AddWithValue("@transactionDate", transactionDate);
                sda.Parameters.AddWithValue("@bankAmount", bankAmount);
                sda.Parameters.AddWithValue("@stockAmount", stockAmount);

                _connectDA.Open();
                sda.ExecuteNonQuery();
                _connectDA.Close();
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
            }

        }
    }
}