﻿/*
*	FILE			: ESMIS_SERVER
*	PROJECT			: ESMIS
*	PROGRAMMER		: BRIAN HINDS, JAMES SIMPSOM, JACKSON RUBY
*	FIRST VERSION	: 2.0.0
*	DESCRIPTION 	: This is the main server that handles client's connection, login, registration, password, data flow 
*/
using System;
using System.ComponentModel;
using System.Configuration;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Windows.Forms;

namespace ESMIS_SERVER
{
    public partial class ESMIS_SERVER : Form
    {
        /// <summary>
        /// Parameters for server connection
        /// </summary>
        private TcpClient client;
        public StreamReader ESMIS_STR;
        public StreamWriter ESMIS_STW;
        public string data_Send;
        public string data_receive;
        private string _hash;
        private string _salt;
        private string _user;

        /// <summary>
        ///Protocol tags for direction data flow
        /// </summary>
        private string _ProtocolA = "USERLOGIN";
        private string _ProtocolB = "USERSALT";
        private string _ProtocolC = "USERHASH";
        private string _ProtocolD = "USERREGISTER";
        private string _ProtocolE = "USERREGISTERPASS";
        private string _ProtocolF = "USERREGISTERFAIL";
        private string _ProtocolG = "USERPASSWORDRESET";
        private string _ProtocolH = "USERRESETVALIDATIONPASS";
        private string _ProtocolHH = "USERRESETVALIDATIONFAIL";
        private string _ProtocolJ = "USERNEWPASSWORD";
        private string _ProtocolK = "USERPASSWORDRESET";
        private string _ProtocolL = "USERPASSWORDRESETFAIL";

        /// <summary>
        ///Server IP configure connection string
        /// </summary>
        private string _ServerIP_Connection = ConfigurationManager.ConnectionStrings["ESMIS_SERVER_IP"].ConnectionString;

        private string clientData;

        /// <summary>
        ///Instantiating the required class needed
        /// </summary>
        private ESMIS_SERVER_ENCR _Encrypt = new ESMIS_SERVER_ENCR();

        private ESMIS_SERVER_LGN userLogin = new ESMIS_SERVER_LGN();
        private ESMIS_SERVER_REG userRegister = new ESMIS_SERVER_REG();
        private ESMIS_DATABASE _DataBase = new ESMIS_DATABASE();
        private ESMIS_SERVER_PWRES _resetPassword = new ESMIS_SERVER_PWRES();

        public ESMIS_SERVER()
        {
            InitializeComponent();
            _Encrypt.getKey();
        }

        private void ESMIS_SERVER_Load(object sender, EventArgs e)
        {
        }

        /*
        *  FUNCTION       : start_server()
        *  DISCRIPTION    : Starts the esmis server
        *  PARAMETER      : none
        *  RETURN         : none
        */

        private void start_server()
        {
            
                TcpListener listener = new TcpListener(IPAddress.Parse(_ServerIP_Connection), int.Parse("8080"));
                listener.Start();
                client = listener.AcceptTcpClient();
                ESMIS_STR = new StreamReader(client.GetStream());
                ESMIS_STW = new StreamWriter(client.GetStream());
                ESMIS_STW.AutoFlush = true;
                connect_Status.Image = Properties.Resources.kiKnky7qT;
                backgroundWorker1.RunWorkerAsync(); // Start receiving data in background
                backgroundWorker2.WorkerSupportsCancellation = true; // Ability to cancel this thread.
          
        }

        /*
        *  FUNCTION       : backgroundWorker1_DoWork
        *  DISCRIPTION    : Handle Receive data from the clients
        *  PARAMETER      : (object sender, DoWorkEventArgs e)
        *  RETURN         : None
        */

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            while (client.Connected)
            {
                try
                {
                    data_receive = ESMIS_STR.ReadLine(); //gets client data
                    clientData = _Encrypt.Decrypt(data_receive, _Encrypt.key); //decrypt data

                    //Login user
                    if (clientData.Contains(_ProtocolA))
                    {
                        //Extract client's data
                        userLogin.userLogin(clientData);
                        //Assign user name for checking
                        _user = userLogin.userName;
                        //Checking Database for user and get their salt
                        _salt = _DataBase.checkLogin(_user);
                        //Display user IP in server window
                        if (!connect_Clients.Items.Contains(userLogin.userIP))
                        { connect_Clients.Items.Add(userLogin.userIP); }
                        //Send their salt to the client
                        sendReply(_salt + "," + _ProtocolB);
                    }

                    if (clientData.Contains(_ProtocolC))
                    {
                        _hash = ExtractHash(clientData);
                        sendReply(_DataBase.checkHash(_hash));
                        ESMIS_LISTEN saveData = new ESMIS_LISTEN();
                        saveData.ShowDialog();
                    }
                    // Register user
                    if (clientData.Contains(_ProtocolD))
                    {
                        bool _IsRegister = true;
                        //Registration data
                        userRegister.userRegistration(clientData);

                        // Checking is user registration successful
                        _IsRegister = userRegister._Success;
                        //Registration successful
                        if (_IsRegister ==false)
                        { sendReply(_ProtocolE); }
                        //User exist or registration failed
                        if (_IsRegister == true)
                        { sendReply(_ProtocolF); }
                    }
                    //comparing hash

                    //Reset password request
                    if (clientData.Contains(_ProtocolG))
                    {
                        bool _Confirm;
                        //Extracting user's reset data for validation
                        _resetPassword.userDataExtraction(clientData);
                        //Resetting user password data in data base
                        _Confirm = _resetPassword._Confirmation;
                        //Informing user of successful change
                        if (_Confirm == true)
                        { sendReply(_ProtocolH); }
                        //Inform user of failure to change
                        if (_Confirm == false)
                        { sendReply(_ProtocolHH); }
                    }
                    //Resetting new password data
                    if (clientData.Contains(_ProtocolJ))
                    {
                        bool _Confirm;
                        _resetPassword.userNewDataExtraction(clientData);
                        _Confirm = _resetPassword._Confirmation;

                        if (_Confirm == true)
                        { sendReply(_ProtocolK); }
                        if (_Confirm == false)
                        { sendReply(_ProtocolL); }
                    }

                   
                }
                catch (Exception x)
                {
                    ServerDisplay.Text = "A client has disconnected";
                    MessageBox.Show(x.Message.ToString());
                }
            }
        }

        /*
        *  FUNCTION       : backgroundWorker2_DoWork
        *  DISCRIPTION    : Handle Send data to the clients
        *  PARAMETER      : (object sender, DoWorkEventArgs e)
        *  RETURN         : None
        */

        private void backgroundWorker2_DoWork(object sender, DoWorkEventArgs e)
        {
            if (client.Connected)
            {
                ESMIS_STW.WriteLine(data_Send);
            }
            else
            {
                ServerDisplay.Text = "A client has disconnected";
            }
            backgroundWorker2.CancelAsync();
        }

        /*
        *  FUNCTION       : sendReply()
        *  DISCRIPTION    : Encrypt and send data to client
        *  PARAMETER      : (string rawData)
        *  RETURN         : None
        */

        private void sendReply(string rawData)
        {
            data_Send = _Encrypt.Encrypt(rawData, _Encrypt.key); ;
            backgroundWorker2.RunWorkerAsync();
        }
        /*
        *  FUNCTION       : ExtractHash
        *  DISCRIPTION    : Extract the hash data from client data
        *  PARAMETER      : (string clientData)
        *  RETURN         : string _hash
        */
        private string ExtractHash(string clientData)
        {
            char delimiter = ',';
            int count = 0;

            string[] substrings = clientData.Split(delimiter);

            foreach (var substring in substrings)
            {
                if (count == 0) { _hash = substring; count++; }
            }
            return _hash;
        }
        /*
        *  FUNCTION       : StartServer_Click
        *  DISCRIPTION    : Start server connection
        *  PARAMETER      : (object sender, EventArgs e)
        *  RETURN         : None
        */
        private void StartServer_Click(object sender, EventArgs e)
        {
            start_server();
        }
    }
}