﻿/*
*	FILE			: ESMIS_SERVER_ENCR.cs
*	PROJECT			: ESMIS
*	PROGRAMMER		: This Class came from Microsoft MSDN
*	FIRST VERSION	: 1.0.0
*	DESCRIPTION 	: This Class encrypt and decrypt data transmission.
*/
using System;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Windows.Forms;

namespace ESMIS_SERVER
{
    internal class ESMIS_SERVER_ENCR
    {
        /*128C Encryption Key*/
        public string key;
        /*Encryption Size*/
        private readonly int EncryptionSize = 64;

        /*
        *  FUNCTION       : getKey()
        *  DISCRIPTION    : Get Public key
        *  PARAMETER      : None
        *  RETURN         : None
        */

        public void getKey()
        {
            var fileStream = new FileStream(@"key.esmis", FileMode.Open, FileAccess.Read);
            using (var streamReader = new StreamReader(fileStream, Encoding.UTF8))
            {
                key = streamReader.ReadToEnd();
            }
        }

        /**
*@METHOD			:Encrypt
*@DESCRIPTION		:This method encrypt the message to be sent
*@PARAMETERS		:string OriginalMsg, string key
*@RETURNS			:WriteMemoryStream(OriginalMsg, ref saltBytes, encryption, ref memoryStream, ref cryptoStream);
*/

        public string Encrypt(string OriginalMsg, string key)
        {
            /*Check for the composed message thats to be encrypted*/
            if (string.IsNullOrEmpty(OriginalMsg))
            {
                MessageBox.Show("Encryption Service did not find any message to encrypt", "Encryption Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            /*Check for the internal encryption key*/
            if (string.IsNullOrEmpty(key))
            {
                throw new ArgumentNullException("Internal encryption key error\nIncorrect Key Size for chosen encryption.");
            }

            using (var keyDerivationFunction = new Rfc2898DeriveBytes(key, EncryptionSize))
            {
                var saltBytes = keyDerivationFunction.Salt;
                var keyBytes = keyDerivationFunction.GetBytes(32);
                var ivBytes = keyDerivationFunction.GetBytes(16);

                using (var aesManaged = new AesManaged())
                {
                    aesManaged.KeySize = 256;

                    using (var encryptor = aesManaged.CreateEncryptor(keyBytes, ivBytes))
                    {
                        MemoryStream memoryStream = null;
                        CryptoStream cryptoStream = null;

                        /*Returns the encryption results*/
                        return WriteMemoryStream(OriginalMsg, ref saltBytes, encryptor, ref memoryStream, ref cryptoStream);
                    }
                }
            }
        }

        /**
        *@METHOD			: Decrypt
        *@DESCRIPTION		: Decrypt incoming strings
        *@PARAMETERS		: (string EncryptedMsg, string key)
        *@RETURNS			: string decrypt data
        */

        public string Decrypt(string EncryptedMsg, string key)
        {
            if (string.IsNullOrEmpty(EncryptedMsg))
            {
                throw new ArgumentNullException("EncryptedMsg");
            }

            if (string.IsNullOrEmpty(key))
            {
                throw new ArgumentNullException("Internal encryption key error\nIncorrect Key Size for chosen encryption.");
            }

            var allTheBytes = Convert.FromBase64String(EncryptedMsg);
            var saltBytes = allTheBytes.Take(EncryptionSize).ToArray();
            var EncryptedMsgBytes = allTheBytes.Skip(EncryptionSize).Take(allTheBytes.Length - EncryptionSize).ToArray();

            using (var keyDerivationFunction = new Rfc2898DeriveBytes(key, saltBytes))
            {
                var keyBytes = keyDerivationFunction.GetBytes(32);
                var ivBytes = keyDerivationFunction.GetBytes(16);

                return DecryptWithAES(EncryptedMsgBytes, keyBytes, ivBytes);
            }
        }

        /**
        *@METHOD			: WriteMemoryStream
        *@DESCRIPTION		: Contains the encryption algorithm
        *@PARAMETERS		: string OriginalMsg, ref byte[] saltBytes, ICryptoTransform encryptor, ref MemoryStream memoryStream, ref CryptoStream cryptoStream)
        *@RETURNS			:
        */

        private string WriteMemoryStream(string OriginalMsg, ref byte[] saltBytes, ICryptoTransform encryptor, ref MemoryStream memoryStream, ref CryptoStream cryptoStream)
        {
            try
            {
                memoryStream = new MemoryStream();

                try
                {
                    cryptoStream = new CryptoStream(memoryStream, encryptor, CryptoStreamMode.Write);

                    using (var streamWriter = new StreamWriter(cryptoStream))
                    {
                        streamWriter.Write(OriginalMsg);
                    }
                }
                finally
                {
                    if (cryptoStream != null)
                    {
                        cryptoStream.Dispose();
                    }
                }

                var EncryptedMsgBytes = memoryStream.ToArray();
                Array.Resize(ref saltBytes, saltBytes.Length + EncryptedMsgBytes.Length);
                Array.Copy(EncryptedMsgBytes, 0, saltBytes, EncryptionSize, EncryptedMsgBytes.Length);

                return Convert.ToBase64String(saltBytes);
            }
            finally
            {
                if (memoryStream != null)
                {
                    memoryStream.Dispose();
                }
            }
        }

        /**
         *@METHOD			: DecryptWithAES
         *@DESCRIPTION		: Contains the decryption algorithm
         *@PARAMETERS		: (byte[] EncryptedMsgBytes, byte[] keyBytes, byte[] ivBytes)
         *@RETURNS			: string
         */

        private static string DecryptWithAES(byte[] EncryptedMsgBytes, byte[] keyBytes, byte[] ivBytes)
        {
            using (var aesManaged = new AesManaged())
            {
                using (var decryptor = aesManaged.CreateDecryptor(keyBytes, ivBytes))
                {
                    MemoryStream memoryStream = null;
                    CryptoStream cryptoStream = null;
                    StreamReader streamReader = null;

                    try
                    {
                        memoryStream = new MemoryStream(EncryptedMsgBytes);
                        cryptoStream = new CryptoStream(memoryStream, decryptor, CryptoStreamMode.Read);
                        streamReader = new StreamReader(cryptoStream);

                        return streamReader.ReadToEnd();
                    }
                    finally
                    {
                        if (memoryStream != null)
                        {
                            memoryStream.Dispose();
                            memoryStream = null;
                        }
                    }
                }
            }
        }
    }
}